==> aolai-0.0.3-sources.jar: cn/ps1/aolai/service/HttpService.java <==
package cn.ps1.aolai.service;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.http.HttpEntity;
import org.apache.http.HttpMessage;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.methods.RequestBuilder;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.ps1.aolai.utils.Const;
import cn.ps1.aolai.utils.Digest;

/**
 * 通过远程 http服务调用的业务操作
 *
 * @author Aolai
 * @verdion 1.0
 * @since 2019年12月21日
 */

@Service
public class HttpService {

    private static Logger LOG = LoggerFactory.getLogger(HttpService.class);

    private static Map<String, String> headers = new HashMap<>();

    static {
        headers.put("Accept", "*/*");
        headers.put("Accept-Encoding", "gzip, deflate, br");
        headers.put("Accept-Language", "zh-CN, zh, en-US");
        headers.put("Cache-Control", "no-cache");
        headers.put("Connection", "keep-alive");
        headers.put("User-Agent", "Chrome/79.0.3945.88");
    }

    // 禁止了自动重定向
    static RequestConfig CONF = RequestConfig.custom()
            .setRedirectsEnabled(false).setConnectTimeout(Const.TIMEOUT)
            .setSocketTimeout(Const.TIMEOUT).build();

    @Autowired
    private UtilsService utilsSvc;

    /**
     * Http请求
     */
    public String doRequest(HttpUriRequest httpUri) {
        CloseableHttpClient http = HttpClientBuilder.create().build();
        CloseableHttpResponse rsp = null;
        String result = null;
        try {
            // 发送请求
            rsp = http.execute(httpUri);
            // 响应结果
            result = getResponse(rsp);
        } catch (Exception e) {
            LOG.error("-> doRequest...");
            e.printStackTrace();
        } finally {
            utilsSvc.close(http, rsp);
        }
        return result;
    }

    /**
     * Http 处理Get请求
     */
    public String doGet(String url, Map<String, String> params) {
        HttpGet httpGet = new HttpGet(setUrl(url, params)); // 添加参数
        httpGet.setConfig(CONF);
        setHeaders(httpGet, url);
        return doRequest(httpGet);
    }

    /**
     * Http 处理Post请求
     */
    public String doPost(String url, Map<String, String> params) {
        HttpEntity entity = null;
        try {
            entity = new UrlEncodedFormEntity(setParams(params), Const.UTF8);
        } catch (Exception e) {
            LOG.error("-> doPost...");
            e.printStackTrace();
        }
        HttpPost httpPost = new HttpPost(url);
        httpPost.setConfig(CONF);
        httpPost.setEntity(entity); // 添加参数
        setHeaders(httpPost, url);
        return doRequest(httpPost);
    }

    /**
     * Http 携带Cookies转发的Post请求
     */
    public String doPost(String url, String cookieHeader,
                         Map<String, String> params) {
        // String cookieHeader = req.getHeader("Cookie");
        if (cookieHeader != null)
            headers.put("Cookie", cookieHeader);
        return doPost(url, params);
    }

    private void setHeaders(HttpMessage msg, String url) {
        msg.addHeader("Referer", utilsSvc.getConf("app.core"));
        msg.addHeader("Host", utilsSvc.getHost(url)[0]); // 域名及端口，或IP地址及端口号
        for (Map.Entry<String, String> e : headers.entrySet()) {
            msg.addHeader(e.getKey(), e.getValue());
        }
    }

    /**
     * 远程post调用，获取json数据（注意：不能用urlConnect代替此方法）
     */
    public Map<String, String> jsonPost(String url, Map<String, String> params) {
        Map<String, String> map = new HashMap<>();
        map.put("jsonstr", utilsSvc.obj2Json(params));
        return utilsSvc.json2Map(doPost(url, map));
    }

    /**
     * 创建参数队列
     */
    private List<NameValuePair> setParams(Map<String, String> params)
            throws UnsupportedEncodingException {
        List<NameValuePair> nvpList = new ArrayList<NameValuePair>();
        if (params != null) {
            for (Map.Entry<String, String> e : params.entrySet()) {
                String key = e.getKey();
                String val = e.getValue();
                if (!utilsSvc.isEmpty(val))
                    nvpList.add(new BasicNameValuePair(key, val));
            }
        }
        return nvpList;
    }

    /**
     * 实现HTTP摘要认证
     */
    private CloseableHttpClient digestHttpClient(String userName,
                                                 String password) {
        // String[] arr = utilsSvc.getHost(url);
        // int port = arr.length > 1 ? Integer.parseInt(arr[1])
        // : AuthScope.ANY_PORT;

        // 摘要认证参数
        UsernamePasswordCredentials creds = new UsernamePasswordCredentials(
                userName, password);
        CredentialsProvider credsProvider = new BasicCredentialsProvider();
        // credsProvider.setCredentials(new AuthScope(arr[0], port), creds);
        credsProvider.setCredentials(AuthScope.ANY, creds);
        HttpClientBuilder http = HttpClientBuilder.create();
        http.setDefaultCredentialsProvider(credsProvider);

        return http.build();
    }

    /**
     * 设置请求参数
     */
    private RequestBuilder setBuilder(HttpEntity entity, String method) {
        RequestBuilder request = null;
        if ("POST".equals(method)) {
            request = RequestBuilder.post();
        } else if ("GET".equals(method)) {
            request = RequestBuilder.get();
        } else if ("PUT".equals(method)) {
            request = RequestBuilder.put();
        } else if ("DELETE".equals(method)) {
            request = RequestBuilder.delete();
        }
        // 添加参数
        return request.setEntity(entity);
    }

    /**
     * HTTP响应
     */
    private String getResponse(HttpResponse rsp) throws Exception {
        String result = null;
        HttpEntity entity = rsp.getEntity();
        int statusCode = rsp.getStatusLine().getStatusCode();
        if (statusCode == 200 && entity != null) {
            result = EntityUtils.toString(entity, Const.UTF8);
            int len = result.length();
            LOG.debug(len > 200 ? result.substring(0, 200) : result);
        } else {
            LOG.error("-> getResponse..." + rsp.getStatusLine());
        }
        return result;
    }

    /**
     * 基于摘要认证的HTTP请求
     *
     * @param userName
     * @param password
     * @param url
     * @param method
     * @return
     */
    public String digestRequest(String userName, String password,
                                HttpUriRequest uriRequest) {
        CloseableHttpClient http = digestHttpClient(userName, password);
        CloseableHttpResponse rsp = null;
        String result = null;
        try {
            // 请求、响应
            rsp = http.execute(uriRequest);
            result = getResponse(rsp);

        } catch (Exception e) {
            LOG.error("-> digestRequest...");
            e.printStackTrace();
        } finally {
            utilsSvc.close(http, rsp);
        }
        return result;
    }

    /**
     * 基于摘要认证的HTTP请求
     *
     * @param userName
     * @param password
     * @param url
     * @return
     */
    public String digestJsonRequest(String userName, String password,
                                    String url, String json, String method) {
        LOG.debug(json);
        StringEntity entity = new StringEntity(json, Const.UTF8);
        entity.setContentType("application/json");
        entity.setContentEncoding(Const.UTF8);
        RequestBuilder builder = setBuilder(entity, method);
        HttpUriRequest request = builder.setUri(url).build();
        json = digestRequest(userName, password, request);
        LOG.debug(json);
        return json;
    }

    public boolean digestDownload(String userName, String password,
                                  String url, String filePath) {
        CloseableHttpClient http = digestHttpClient(userName, password);
        CloseableHttpResponse rsp = null;
        boolean result = false;
        try {
            // 请求、响应
            HttpGet request = new HttpGet(url);
            rsp = http.execute(request);
            HttpEntity entity = rsp.getEntity();
            InputStream ins = entity.getContent();
            FileOutputStream fos = new FileOutputStream(filePath, false);
            int data = ins.read();
            while (data != -1) {
                fos.write(data);
                data = ins.read();
            }
            fos.close();
            EntityUtils.consume(entity);
            return true;
        } catch (Exception e) {
            LOG.error("-> digestDownload...");
            e.printStackTrace();
        } finally {
            utilsSvc.close(http, rsp);
        }
        return result;
    }

    /**
     * 读取输入流，返回字符串
     *
     * @param in
     * @return
     */
    private String rspAsString(InputStream in) {
        StringBuilder strBuf = new StringBuilder();
        BufferedReader reader = null;
        try {
            reader = new BufferedReader(new InputStreamReader(in, Const.UTF8));
            String line;
            while ((line = reader.readLine()) != null) {
                strBuf.append(line);
            }
        } catch (IOException e) {
            LOG.error("-> rspAsString...");
            e.printStackTrace();
        } finally {
            utilsSvc.close(reader);
        }
        return strBuf.toString();
    }

    private HttpURLConnection setProperty(HttpURLConnection conn) {
        conn.setConnectTimeout(Const.TIMEOUT); // 设置相应请求时间：120秒
        conn.setReadTimeout(Const.TIMEOUT); // 设置读取超时时间：120秒
        conn.setUseCaches(false); // 默认是true
        conn.setDoOutput(true); // 默认是false
        conn.setDoInput(true); // 默认是true
        for (Map.Entry<String, String> e : headers.entrySet()) {
            conn.setRequestProperty(e.getKey(), e.getValue());
        }
        return conn;
    }

    /**
     * 通过HTTP方法请求数据
     */
    public String urlConnect(String url, byte[] bytes, String method) {
        String result = "";
        HttpURLConnection conn = null;
        try {
            LOG.debug("-> urlConnect..." + url);
            URL httpUrl = new URL(url);
            conn = (HttpURLConnection) httpUrl.openConnection();
            setProperty(conn).setRequestMethod(method);

            conn.getOutputStream().write(bytes); // 输出流

            // PrintWriter out = new PrintWriter(conn.getOutputStream());
            // out.print(s); // 发送请求参数
            // out.flush(); // 输出流的缓冲

            // 发送请求
            if (conn.getResponseCode() == 200 || conn.getResponseCode() == 204) {
                result = rspAsString(conn.getInputStream());
            } else {
                result = rspAsString(conn.getErrorStream());
                LOG.error("-> urlConnect..." + result);
            }
        } catch (Exception e) {
            LOG.error("-> urlConnect...");
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 向指定 URL发送GET请求
     */
    public String urlConnect(String url, Map<String, String> params) {
        return urlConnect(setUrl(url, params), "".getBytes(), "GET");
    }

    /**
     * 向指定 URL发送POST请求
     */
    public String urlConnect(String url, String str, String method) {
        String result = "";
        if (str != null)
            result = urlConnect(url, str.getBytes(Const.CHARSET), method);
        return result;
    }

    /**
     * 拼接请求参数
     *
     * @param url
     * @param params
     * @return
     */
    private String setUrl(String url, Map<String, String> params) {
        if (params != null && params.size() > 0) {
            String args = "";
            for (Map.Entry<String, String> e : params.entrySet()) {
                args += "&" + e.getKey() + "=" + Digest.urlEncode(e.getValue());
            }
            if (args.length() > 0)
                url += args.substring(1);
        }
        return url;
    }

    /** 以下通过远程调用业务逻辑，获取信息 */

    /**
     * 构造函数（为了测试用）
     */
    public HttpService() {
        utilsSvc = new UtilsService();
    }

    /**
     * 直接转发，远程调用用户验证
     */
    public Map<String, Object> forward(String ws, HttpServletRequest req) {
        String jsonStr = ssoGetData(ws, utilsSvc.certParams(req));
        return utilsSvc.json2Map(jsonStr);
    }

    // sso请求，非sso可以直接使用doPsot
    public Map<String, String> ssoPost(String ws, Map<String, String> params) {
        // 单点登录的远程服务地址：http://sso.ps1.cn/aolai/s
        final String ssoUrl = utilsSvc.getConf("sso.url");
        return utilsSvc.json2Map(callRemote(ssoUrl + ws, params));
    }

    /**
     * 远程调用乡镇所辖村信息
     */
    public String ssoGetData(String ws, Map<String, String> params) {
        // 单点登录的远程服务地址：http://sso.ps1.cn/aolai/s
        final String ssoUrl = utilsSvc.getConf("sso.url");
        return callRemote(ssoUrl + ws, params);
    }

    /**
     * 远程服务调用
     */

    // 增加了校验的远程http请求，不加校验可以直接使用doPost
    public String callRemote(String url, Map<String, String> params) {
        LOG.info("-> callRemote..." + url);
        // 返回远程认证调用SSO结果，如：/userAuth
        final String appKey = utilsSvc.getConf("app.key");
        params.put("k", appKey); // 如：www.ps1.cn
        return doPost(url, params);
    }

    /**
     * 以下调用当前应用管理服务，获取信息
     */
    public String getMaster(String ws, Map<String, String> params) {
        final String mstUri = utilsSvc.getConf("app.core")
                + utilsSvc.getConf("app.main") + "/w";
        // return utilsSvc.json2Map(doPost(mstUri + ws, params));
        return callRemote(mstUri + ws, params);
    }

}

==> aolai-1.0.0-sources.jar: cn/ps1/aolai/service/HttpService.java <==
package cn.ps1.aolai.service;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.http.HttpEntity;
import org.apache.http.HttpMessage;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.methods.RequestBuilder;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.ps1.aolai.utils.Const;
import cn.ps1.aolai.utils.Digest;

/**
 * 通过远程 http服务调用的业务操作
 *
 * @author Aolai
 * @verdion 1.0
 * @since 2019年12月21日
 */

@Service
public class HttpService {

    private static Logger LOG = Logger.getLogger(HttpService.class);

    private static Map<String, String> headers = new HashMap<>();

    static {
        headers.put("Accept", "*/*");
        headers.put("Accept-Encoding", "gzip, deflate, br");
        headers.put("Accept-Language", "zh-CN, zh, en-US");
        headers.put("Cache-Control", "no-cache");
        headers.put("Connection", "keep-alive");
        headers.put("User-Agent", "Chrome/79.0.3945.88");
    }

    // 禁止了自动重定向
    static RequestConfig CONF = RequestConfig.custom()
            .setRedirectsEnabled(false).setConnectTimeout(Const.TIMEOUT)
            .setSocketTimeout(Const.TIMEOUT).build();

    @Autowired
    private UtilsService utilsSvc;

    /**
     * Http请求
     */
    public String doRequest(HttpUriRequest httpUri) {
        CloseableHttpClient http = HttpClientBuilder.create().build();
        CloseableHttpResponse rsp = null;
        String result = null;
        try {
            // 发送请求
            rsp = http.execute(httpUri);
            // 响应结果
            result = getResponse(rsp);
        } catch (Exception e) {
            LOG.error("-> doRequest...");
            e.printStackTrace();
        } finally {
            utilsSvc.close(http, rsp);
        }
        return result;
    }

    /**
     * Http 处理Get请求
     */
    public String doGet(String url, Map<String, String> parms) {
        HttpGet httpGet = new HttpGet(setUrl(url, parms)); // 添加参数
        httpGet.setConfig(CONF);
        setHeaders(httpGet, url);
        return doRequest(httpGet);
    }

    /**
     * Http 处理Post请求
     */
    public String doPost(String url, Map<String, String> parms) {
        HttpEntity entity = null;
        try {
            entity = new UrlEncodedFormEntity(setParms(parms), Const.UTF8);
        } catch (Exception e) {
            LOG.error("-> doPost...");
            e.printStackTrace();
        }
        HttpPost httpPost = new HttpPost(url);
        httpPost.setConfig(CONF);
        httpPost.setEntity(entity); // 添加参数
        setHeaders(httpPost, url);
        return doRequest(httpPost);
    }

    /**
     * Http 携带Cookies转发的Post请求
     */
    public String doPost(String url, String cookieHeader,
                         Map<String, String> parms) {
        // String cookieHeader = req.getHeader("Cookie");
        if (cookieHeader != null)
            headers.put("Cookie", cookieHeader);
        return doPost(url, parms);
    }

    private void setHeaders(HttpMessage msg, String url) {
        msg.addHeader("Referer", utilsSvc.getConf("app.core"));
        msg.addHeader("Host", utilsSvc.getHost(url)[0]); // 域名及端口，或IP地址及端口号
        for (Map.Entry<String, String> e : headers.entrySet()) {
            msg.addHeader(e.getKey(), e.getValue());
        }
    }

    /**
     * 远程post调用，获取json数据（注意：不能用urlConnect代替此方法）
     */
    public Map<String, String> jsonPost(String url, Map<String, String> parms) {
        Map<String, String> map = new HashMap<>();
        map.put("jsonstr", utilsSvc.obj2Json(parms));
        return utilsSvc.json2Map(doPost(url, map));
    }

    /**
     * 创建参数队列
     */
    private List<NameValuePair> setParms(Map<String, String> parms)
            throws UnsupportedEncodingException {
        List<NameValuePair> nvpList = new ArrayList<NameValuePair>();
        if (parms != null) {
            for (Map.Entry<String, String> e : parms.entrySet()) {
                String key = e.getKey();
                String val = e.getValue();
                if (!utilsSvc.isEmpty(val))
                    nvpList.add(new BasicNameValuePair(key, val));
            }
        }
        return nvpList;
    }

    /**
     * 实现HTTP摘要认证
     */
    private CloseableHttpClient digestHttpClient(String userName,
                                                 String password) {
        // String[] arr = utilsSvc.getHost(url);
        // int port = arr.length > 1 ? Integer.parseInt(arr[1])
        // : AuthScope.ANY_PORT;

        // 摘要认证参数
        UsernamePasswordCredentials creds = new UsernamePasswordCredentials(
                userName, password);
        CredentialsProvider credsProvider = new BasicCredentialsProvider();
        // credsProvider.setCredentials(new AuthScope(arr[0], port), creds);
        credsProvider.setCredentials(AuthScope.ANY, creds);
        HttpClientBuilder http = HttpClientBuilder.create();
        http.setDefaultCredentialsProvider(credsProvider);

        return http.build();
    }

    /**
     * 设置请求参数
     */
    private RequestBuilder setBuilder(HttpEntity entity, String method) {
        RequestBuilder request = null;
        if ("POST".equals(method)) {
            request = RequestBuilder.post();
        } else if ("GET".equals(method)) {
            request = RequestBuilder.get();
        } else if ("PUT".equals(method)) {
            request = RequestBuilder.put();
        } else if ("DELETE".equals(method)) {
            request = RequestBuilder.delete();
        }
        // 添加参数
        return request.setEntity(entity);
    }

    /**
     * HTTP响应
     */
    private String getResponse(HttpResponse rsp) throws Exception {
        String result = null;
        HttpEntity entity = rsp.getEntity();
        int statusCode = rsp.getStatusLine().getStatusCode();
        if (statusCode == 200 && entity != null) {
            result = EntityUtils.toString(entity, Const.UTF8);
            int len = result.length();
            LOG.debug(len > 200 ? result.substring(0, 200) : result);
        } else {
            LOG.error(rsp.getStatusLine());
        }
        return result;
    }

    /**
     * 基于摘要认证的HTTP请求
     *
     * @param userName
     * @param password
     * @param url
     * @param method
     * @return
     */
    public String digestRequest(String userName, String password,
                                HttpUriRequest uriRequest) {
        CloseableHttpClient http = digestHttpClient(userName, password);
        CloseableHttpResponse rsp = null;
        String result = null;
        try {
            // 请求、响应
            rsp = http.execute(uriRequest);
            result = getResponse(rsp);

        } catch (Exception e) {
            LOG.error("-> digestRequest...");
            e.printStackTrace();
        } finally {
            utilsSvc.close(http, rsp);
        }
        return result;
    }

    /**
     * 基于摘要认证的HTTP请求
     *
     * @param userName
     * @param password
     * @param url
     * @return
     */
    public String digestJsonRequest(String userName, String password,
                                    String url, String json, String method) {
        LOG.debug(json);
        StringEntity entity = new StringEntity(json, Const.UTF8);
        entity.setContentType("application/json");
        entity.setContentEncoding(Const.UTF8);
        RequestBuilder builder = setBuilder(entity, method);
        HttpUriRequest request = builder.setUri(url).build();
        json = digestRequest(userName, password, request);
        LOG.debug(json);
        return json;
    }

    public boolean digestDownload(String userName, String password,
                                  String url, String filePath) {
        CloseableHttpClient http = digestHttpClient(userName, password);
        CloseableHttpResponse rsp = null;
        boolean result = false;
        try {
            // 请求、响应
            HttpGet request = new HttpGet(url);
            rsp = http.execute(request);
            HttpEntity entity = rsp.getEntity();
            InputStream ins = entity.getContent();
            FileOutputStream fos = new FileOutputStream(filePath, false);
            int data = ins.read();
            while (data != -1) {
                fos.write(data);
                data = ins.read();
            }
            fos.close();
            EntityUtils.consume(entity);
            return true;
        } catch (Exception e) {
            LOG.error("-> digestDownload...");
            e.printStackTrace();
        } finally {
            utilsSvc.close(http, rsp);
        }
        return result;
    }

    /**
     * 读取输入流，返回字符串
     *
     * @param in
     * @return
     */
    private String rspAsString(InputStream in) {
        StringBuilder strBuf = new StringBuilder();
        BufferedReader reader = null;
        try {
            reader = new BufferedReader(new InputStreamReader(in, Const.UTF8));
            String line;
            while ((line = reader.readLine()) != null) {
                strBuf.append(line);
            }
        } catch (IOException e) {
            LOG.error("-> rspAsString...");
            e.printStackTrace();
        } finally {
            utilsSvc.close(reader);
        }
        return strBuf.toString();
    }

    private HttpURLConnection setProperty(HttpURLConnection conn) {
        conn.setConnectTimeout(Const.TIMEOUT); // 设置相应请求时间：120秒
        conn.setReadTimeout(Const.TIMEOUT); // 设置读取超时时间：120秒
        conn.setUseCaches(false); // 默认是true
        conn.setDoOutput(true); // 默认是false
        conn.setDoInput(true); // 默认是true
        for (Map.Entry<String, String> e : headers.entrySet()) {
            conn.setRequestProperty(e.getKey(), e.getValue());
        }
        return conn;
    }

    /**
     * 通过HTTP方法请求数据
     */
    public String urlConnect(String url, byte[] bytes, String method) {
        String result = "";
        HttpURLConnection conn = null;
        try {
            LOG.debug("-> urlConnect..." + url);
            URL httpUrl = new URL(url);
            conn = (HttpURLConnection) httpUrl.openConnection();
            setProperty(conn).setRequestMethod(method);

            conn.getOutputStream().write(bytes); // 输出流

            // PrintWriter out = new PrintWriter(conn.getOutputStream());
            // out.print(s); // 发送请求参数
            // out.flush(); // 输出流的缓冲

            // 发送请求
            if (conn.getResponseCode() == 200 || conn.getResponseCode() == 204) {
                result = rspAsString(conn.getInputStream());
            } else {
                result = rspAsString(conn.getErrorStream());
                LOG.error("-> urlConnect..." + result);
            }
        } catch (Exception e) {
            LOG.error("-> urlConnect...");
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 向指定 URL发送GET请求
     */
    public String urlConnect(String url, Map<String, String> parms) {
        return urlConnect(setUrl(url, parms), "".getBytes(), "GET");
    }

    /**
     * 向指定 URL发送POST请求
     */
    public String urlConnect(String url, String str, String method) {
        String result = "";
        if (str != null)
            result = urlConnect(url, str.getBytes(Const.CHARSET), method);
        return result;
    }

    /**
     * 拼接请求参数
     *
     * @param url
     * @param parms
     * @return
     */
    private String setUrl(String url, Map<String, String> parms) {
        if (parms != null && parms.size() > 0) {
            String args = "";
            for (Map.Entry<String, String> e : parms.entrySet()) {
                args += "&" + e.getKey() + "=" + Digest.urlEncode(e.getValue());
            }
            if (args.length() > 0)
                url += args.substring(1);
        }
        return url;
    }

    /** 以下通过远程调用业务逻辑，获取信息 */

    /**
     * 构造函数（为了测试用）
     */
    public HttpService() {
        utilsSvc = new UtilsService();
    }

    /**
     * 直接转发，远程调用用户验证
     */
    public Map<String, Object> forward(String ws, HttpServletRequest req) {
        String jsonStr = ssoGetData(ws, utilsSvc.certParms(req));
        return utilsSvc.json2Map(jsonStr);
    }

    // sso请求，非sso可以直接使用doPsot
    public Map<String, String> ssoPost(String ws, Map<String, String> parms) {
        // 单点登录的远程服务地址：http://sso.ps1.cn/aolai/s
        final String ssoUrl = utilsSvc.getConf("sso.url");
        return utilsSvc.json2Map(callRemote(ssoUrl + ws, parms));
    }

    /**
     * 远程调用乡镇所辖村信息
     */
    public String ssoGetData(String ws, Map<String, String> parms) {
        // 单点登录的远程服务地址：http://sso.ps1.cn/aolai/s
        final String ssoUrl = utilsSvc.getConf("sso.url");
        return callRemote(ssoUrl + ws, parms);
    }

    /**
     * 远程服务调用
     */

    // 增加了校验的远程http请求，不加校验可以直接使用doPost
    public String callRemote(String url, Map<String, String> parms) {
        LOG.info("-> callRemote..." + url);
        // 返回远程认证调用SSO结果，如：/userAuth
        final String appKey = utilsSvc.getConf("app.key");
        parms.put("k", appKey); // 如：www.ps1.cn
        return doPost(url, parms);
    }

    /**
     * 以下调用当前应用管理服务，获取信息
     */
    public String getMaster(String ws, Map<String, String> parms) {
        final String mstUri = utilsSvc.getConf("app.core")
                + utilsSvc.getConf("app.main") + "/w";
        // return utilsSvc.json2Map(doPost(mstUri + ws, parms));
        return callRemote(mstUri + ws, parms);
    }

}
